package org.greenstone.gatherer.collection;

import java.io.*;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.CommandTokenizer;
import org.greenstone.gatherer.msm.MSMUtils;
import org.greenstone.gatherer.util.Codec;
import org.greenstone.gatherer.util.StaticStrings;
import org.greenstone.gatherer.util.Utility;
import org.w3c.dom.*;

/** This class provides access to a collection configuration file. This version accepts either a valid xml document or a historical collect.cfg file. */
public class CollectionConfiguration 
    implements Comparable {

    private Element creator_element;
    private Element description_element;
    private Element maintainer_element;
    private Element name_element;
    private File file;
    private String build_type;
    private String creator;
    private String description;
    private String maintainer;
    private String name;
    private String search_types;

    public CollectionConfiguration(File file) {
	this.file = file;
	try {
	    String filename = file.getName().toLowerCase();
	    if(filename.endsWith(".xml")) {

	    }
	    else if(filename.endsWith(".cfg")) {
		FileReader fr = new FileReader(file);
		BufferedReader br = new BufferedReader(fr);
		String command = null;
		while((command = br.readLine()) != null) {
		    if(command.length() > 0) {
			// We have to test the end of command for the special character '\'. If found, remove it and append the next line, then repeat.
			while(command.trim().endsWith("\\")) {
			    command = command.substring(0, command.lastIndexOf("\\"));
			    String next_line = br.readLine();
			    if(next_line != null) {
				command = command + next_line;
			    }
			    next_line = null;
			}
	    		CommandTokenizer tokenizer = new CommandTokenizer(command);
			if(tokenizer.countTokens() >= 2) {
			    String temp = tokenizer.nextToken();
			    if(temp.equalsIgnoreCase("creator")) {
				creator = tokenizer.nextToken();
			    }
			    else if(temp.equalsIgnoreCase("maintainer")) {
				maintainer = tokenizer.nextToken();
			    }
			    else if (temp.equalsIgnoreCase("buildtype")) {
				build_type = tokenizer.nextToken();
			    }
			    else if (temp.equalsIgnoreCase("searchtype")) {
				search_types = tokenizer.nextToken();
				temp = tokenizer.nextToken();
				if (temp!=null && !temp.equals("")) {
				    search_types += ","+temp;
				}
			    }
			    else if(temp.equalsIgnoreCase("collectionmeta")) {
				String meta_type = tokenizer.nextToken();
				temp = tokenizer.nextToken();
				// check for language
				String language = "en"; // assume the no-lang ones are english, but we shouldn't really do this.
				if(temp.startsWith("[") && temp.endsWith("]")) {
				    language = temp.substring(temp.indexOf("=") + 1, temp.length() - 1); 
				    temp = tokenizer.nextToken();
				}
								
				// now we need to read in the whole of the entry - may span multiple lines, may be surrounded by single or double quotes
				String start_string = temp.substring(0,1);
				if (start_string.equals("\"") || start_string.equals("\'")) {
				    if (temp.endsWith(start_string) && temp.length() != 1) {
					temp = temp.substring(1, temp.length() - 1);
				    }
				    else {
					StringBuffer value_raw = new StringBuffer(temp.substring(1));
					// add the new line back in
					value_raw.append(StaticStrings.NEW_LINE_CHAR);
					int pos = value_raw.indexOf(start_string);
					int old_pos = 0;
					while (pos != -1 && value_raw.charAt(pos-1)=='\\') {
					    old_pos = pos+1;
					    pos = value_raw.indexOf(start_string, old_pos);
					}
					while(pos == -1) {
					    String next_line = br.readLine();
					    if(next_line != null) {
						value_raw.append(next_line);
						value_raw.append(StaticStrings.NEW_LINE_CHAR);
					    }
					    next_line = null;
					    pos = value_raw.indexOf(start_string, old_pos);
					    while (pos != -1 && value_raw.charAt(pos-1)=='\\') {
						old_pos = pos+1;
						pos = value_raw.indexOf(start_string, old_pos);
					    }
					}

					temp = value_raw.substring(0, value_raw.lastIndexOf(start_string));
					value_raw = null;
				    }
				}
				
				// now we can work out which coll meta we are dealing with
				if (meta_type.equalsIgnoreCase("collectionname")) {
				    if (name==null || language.equalsIgnoreCase(Gatherer.dictionary.getLanguage())) {
					name= Utility.decodeGreenstone(temp);
					// should we use Codec here too?
					
				    }
				    
				} else if (meta_type.equalsIgnoreCase("collectionextra")) {
				    if (description == null|| language.equalsIgnoreCase(Gatherer.dictionary.getLanguage())) {
					//description = Utility.decodeGreenstone(temp);
					// or should we use codec??
					description = Codec.transform(temp, Codec.GREENSTONE_TO_TEXT);
				    }
				}
				language = null;
				meta_type = null;
			    } // end of coll meta bit
			    temp = null;
			} // if num tokens >= 2
			tokenizer = null;
		    } // if command.length > 0
		} // while
		// just check a couple of things
		if (search_types != null && build_type==null) {
		    build_type = "mgpp";
		} 
		command = null;
		br.close();
		br = null;
		fr.close();
		fr = null;
	    } // cfg file
	    ///ystem.err.println("Parsed collect.cfg");
	    ///ystem.err.println("name        = " + name);
	    ///ystem.err.println("creator     = " + creator);
	    ///ystem.err.println("maintainer  = " + maintainer);
	    ///ystem.err.println("description = " + description);
	}
	catch(Exception error) {
	    Gatherer.println("Error in CollectionConfiguration.<init>(): " + error);
	    Gatherer.printStackTrace(error);
	}
    }

    public int compareTo(Object other) {
	if(other == null) {
	    return -1;
	}
	return toString().compareTo(other.toString());
    }

    public boolean equals(Object other) {
	return (compareTo(other) == 0);
    }

    public String getBuildType() {
	String result = "";
	    
	if (build_type != null) {
	    result = build_type;
	}
	return result;
    }

    public String getCreator() {
	String result = "";
	if(creator_element != null) {
	    result = MSMUtils.getValue(creator_element);
	}
	else if(creator != null) {
	    result = creator;
	}
	return result;
    }

    public String getDescription() {
	String result = "";
	if(description_element != null) {
	    result = MSMUtils.getValue(description_element);
	}
	else if(description != null) {
	    result = description;
	}
	return result;
    }

	public File getFile() {
		return file;
	}

    public String getMaintainer() {
	String result = "";
	if(maintainer_element != null) {
	    result = MSMUtils.getValue(maintainer_element);
	}
	else if(maintainer != null) {
	    result = maintainer;
	}
	return result;
    }

    public String getName() {
	String result = StaticStrings.ERROR_STR;
	if(name_element != null) {
	    result = MSMUtils.getValue(name_element);
	}
	else if(name != null) {
	    result = name;
	}
	return result;
    }

    public String getSearchTypes() {
	String result = "";
	    
	if (search_types != null) {
	    result = search_types;
	}
	return result;
    }

    /** Retrieve the short name for this collection which, given this current file is in <col_name>/etc/collect.cfg, is the name of this file's parent file's parent.
     * @return the short name of this collection as a String
     */
    public String getShortName() {
	return file.getParentFile().getParentFile().getName();
    }

    public void setName(String name) {
	/** @todo */
    }

    /** Display the title for this collection. */
    public String toString() {
	return getShortName() + " - \"" + getName() + "\"";
    }
}
